#include "libpcap.h"

int cap_fd=-1;

int lpcap_init(void)
{
	struct pcap_file_header phdr;
	int fe;

	if ( !flag(FLAG_LIBPCAP) ) return 1;

	fe = lpcap_file_exists(globals.pcapfile);

	cap_fd = open(globals.pcapfile, O_WRONLY | O_CREAT, 00600);
	if ( cap_fd < 0 ) {
		perror("opening tcpdump file" CRLF);
		return 0;
	}


	/* Setup the header if the file didnt
	 * already exist. */
	if ( !fe ) {
		phdr.magic = 0xa1b2c3d4; /* Stolen from snort (??) */
		phdr.version_major = PCAP_VERSION_MAJOR;
		phdr.version_minor = PCAP_VERSION_MINOR;
		phdr.thiszone = 0;
		phdr.sigfigs = 0;
		phdr.snaplen = BUFFER_SIZE;
		phdr.linktype = DLT_RAW;

		if ( write(cap_fd, &phdr, sizeof(phdr)) < 0 ) {
			perror("writing pcap header");
			return 0;
		}
		
		fsync(cap_fd);
	}

	return 1;
}

/* is this file a valid pcap file? If not, clobber it. 
 * FIXME: Is clobbering the file bad behaivour? */
int lpcap_file_exists(const char *fn)
{
	int tmp_fd;
	struct stat sfile;
	struct pcap_file_header phdr;
	
	if ( stat(fn, &sfile) != 0 ) return 0;

	tmp_fd = open(fn, O_RDONLY);
	if ( tmp_fd < 0 ) return 0;

	if ( read(tmp_fd, &phdr, sizeof(phdr)) < sizeof(phdr) ) {
		close(tmp_fd);
		return 0;
	}

	close(tmp_fd);

	if ( ! ((phdr.magic == 0xa1b2c3d4) && 
		(phdr.linktype == DLT_RAW)) ) return 0;
	
	return 1;
}

void lpcap_close(void)
{
	if ( !flag(FLAG_LIBPCAP) ) return;
	if ( cap_fd >= 0 ) {
		fsync(cap_fd);
		close(cap_fd);
		cap_fd = -1;
	}
}

void lpcap_dump ( p_packet p )
{
	struct pcap_pkthdr phdr;

	phdr.caplen	= p->data.netlink.len-sizeof(p->data.netlink);
	phdr.len 	= ntohs(p->ip->tot_len);
	gettimeofday( (struct timeval *)&phdr.ts, NULL );

	write(cap_fd, &phdr, sizeof(phdr) );
	write(cap_fd, p->ip, phdr.caplen );
	fsync(cap_fd);
}
