#include "util.h"

char *strtoupper(char *in)
{
	char *tmp=in;
	
	/* AUDIT: As long as get_proto() is the only caller
	 * of this function, bounds checking is not needed, as
	 * get_proto() gives us data straight from getprotobynumber()
	 */
	do {
		*tmp = toupper(*tmp);
	}while ( *(tmp++) != '\0' );
	
	return in;
}

const char *get_tcpflags(struct tcphdr *tcp)
{
	static char buf[16];
	char *b=(char *)&buf;

	memset(b, 0, 16);
	
	if ( tcp->ack ) strncat(b, "A", 1);
	if ( tcp->psh ) strncat(b, "P", 1);
	if ( tcp->rst ) strncat(b, "R", 1);
	if ( tcp->syn ) strncat(b, "S", 1);
	if ( tcp->fin ) strncat(b, "F", 1);
	if ( tcp->urg ) strncat(b, "U", 1);

	return b;
}

const char *get_proto(int p)
{
	struct protoent *proto;

	if ( flag(FLAG_CHROOT) ) {
		switch (p) {
			case 1: return "ICMP";
			case 2: return "IGMP";
			case 6: return "TCP";
			case 17: return "UDP";
			case 41: return "IPv6";
			case 43: return "IPv6-route";
			case 44: return "IPv6-frag";
			case 50: return "IPv6-crypt";
			case 51: return "IPv6-auth";
			case 58: return "IPv6-icmp";
			case 59: return "IPv6-nonxt";
			case 60: return "IPv6-opts";
			case 94: return "IPIP";
			case 98: return "ENCAP";
		}
	}
	
	proto = getprotobynumber(p);

	/* AUDIT: Callers no longer need check for NULL */
	if (proto)
		return strtoupper(proto->p_name);
	else
		return "(NOT KNOWN)";
}

int format_ip ( u_long ip, char *txt )
{
	struct in_addr addr;
	addr.s_addr=ip;
	strncpy( txt, inet_ntoa(addr), 127 );
	return 1;
}

/* this is disgusting */
int print_icmp_type(u_char type, u_char code)
{
	switch ( type ) {
		case 0:
			print_out("ICMP_ECHO_REPLY");
			return 1;
		case 3:
			switch ( code ){
				case 0:
					print_out("ICMP_NET_UNREACH");
					return 1;
				case 1:
					print_out("ICMP_HOST_UNREACH");
					return 1;
				case 2:
					print_out("ICMP_PROTO_UNREACH");
					return 1;
				case 3:
					print_out("ICMP_PORT_UNREACH");
					return 1;
				case 4:
					print_out("ICMP_FRAG_NEEDED_BUT_DF_SET");
					return 1;
				case 5:
					print_out("ICMP_SOURCE_ROUTE_FAILED");
					return 1;
				case 6:
					print_out("ICMP_NET_UNKNOWN");
					return 1;
				case 7:
					print_out("ICMP_HOST_UNKNOWN");
					return 1;
				case 8:
					print_out("ICMP_SOURCE_HOST_ISOLATED");
					return 1;
				case 9:
					print_out("ICMP_NET_ADMIN_PROHIBITED");
					return 1;
				case 10:
					print_out("ICMP_HOST_ADMIN_PROHIBITED");
					return 1;
				case 11:
					print_out("ICMP_NET_UNREACH_TOS");
					return 1;
				case 12:
					print_out("ICMP_HOST_UNREACH_TOS");
					return 1;
				case 13:
					print_out("ICMP_COMMS_FILTERED");
					return 1;
				case 14:
					print_out("ICMP_PRECEDENCE_VIOLATION");
					return 1;
				case 15:
					print_out("ICMP_PRECEDENCE_CUTOFF");
					return 1;
				default:
					print_out("ICMP_DEST_UNCREACH");
					return 1;
			}
		case 4:
			print_out("ICMP_SOURCE_QUENCH");
			return 1;
		case 5:
			switch ( code ){
				case 0:
					print_out("ICMP_REDIRECT_NET");
					return 1;
				case 1:
					print_out("ICMP_REDIRECT_HOST");
					return 1;
				case 2:
					print_out("ICMP_REDIRECT_TOS_NET");
					return 1;
				case 3:
					print_out("ICMP_REDIRECT_TOS_HOST");
					return 1;
				default:
					print_out("ICMP_REDIRECT");
					return 1;
			}
		case 8:
			print_out("ICMP_ECHO_REQUEST");
			return 1;
		case 9:
			print_out("ICMP_ROUTER_ADVERTISMENT");
			return 1;
		case 10:
			print_out("ICMP_ROUTER_SOLICITATION");
			return 1;
		case 11:
			if ( code == 0 ){
				print_out("ICMP_TTL_EXCEEDED");
			}else if ( code == 1 ){
				print_out("ICMP_TTL_EXCEEDED_IN_REASSEMBLY");
			}else{
				print_out("ICMP_TIME_EXCEED");
			}return 1;
		case 12:
			if ( code == 0 ){
				print_out("ICMP_IP_HEADER_BAD");
			}else if ( code == 1 ){
				print_out("ICMP_IP_OPTION_MISSING");
			}else{
				print_out("ICMP_BAD_PARAM");
			}return 1;
		case 13:
			print_out("ICMP_TIMESTAMP_REQUEST");
			return 1;
		case 14:
			print_out("ICMP_TIMESTAMP_REPLY");
			return 1;
		case 15:
			print_out("ICMP_INFO_REQUEST");
			return 1;
		case 16:
			print_out("ICMP_INFO_REPLY");
			return 1;
		case 17:
			print_out("ICMP_MASK_REQUEST");
			return 1;
		case 18:
			print_out("ICMP_MASK_REPLY");
			return 1;
	}

	print_out("ICMP_UNKNOWN (%.2x:%.2x)", type, code);
	return 1;
}
